/*----------------------------------------------------------------*/
/* FrExtract.c             by B. Mours LAPP   15/1/98             */
/*                                                                */
/* This Matlab mex file extract from a frame file the data for    */ 
/*   one ADC channel (this is for Matlab version 5)               */
/*                                                                */
/* The input arguments are:                                       */
/*   1) file name                                                 */
/*   2) ADC name                                                  */
/*   3) (optional) first frame (default= first frame in the file) */
/*   4) (optional) number of frame (default = 1 Frames)           */
/*                                                                */
/* Returned matlab data:                                          */
/*   1) ADC data (time serie)                                     */
/*   2) (optional) time values relative to the first data point   */
/*   3) (optional) frequency values (for FFT's)                   */
/*   4) (optional) GPS starting time (in second.nanosec)          */
/*   5) (optional) starting time as a string                      */
/*   6) (optional) ADC comment as a string                        */
/*   7) (optional) ADC unit as a string                           */
/*   8) (optional) additional info: it is a 9 words vector which  */
/*       content:crate #, channel#, nBits#, biais, slope,         */
/*               sampleRate, timeOffset(S.N), fShift, overRange   */
/*      All this values are stored as double                      */
/*----------------------------------------------------------------*/
#include "mex.h"
#include "FrameL.h"

void  mexFunction( int nlhs,       mxArray *plhs[],
                   int nrhs, const mxArray *prhs[])
{struct FrFile *iFile;
 struct FrameH *frame;
 struct FrAdcData *adc;
 struct FrVect *vect;
 int   status, first, nFrames;
 long buffSize, debugLevel, i, nBin, nData, index;
 double *data, dt, df;
 char *buff, *name, fileName[100],adcName[100], stringTime[30];
 time_t tp;

    /*--------------- check arguments ---------------------*/

 if(nrhs<2)
    {mexErrMsgTxt("Please provide the file name and ADC name as argument\n");
     return;}

 if(!mxIsChar(prhs[0]))
    {mexErrMsgTxt("The first arguement is not a filename\n");
     return;}
  
 if(!mxIsChar(prhs[1]))
    {mexErrMsgTxt("The second arguement is not the adc name \n");
     return;}
  
 if(nlhs==0)
    {mexErrMsgTxt("Please provide at least one output argument\n");
     return;}

 if(nrhs > 2)
     {data = mxGetPr(prhs[2]);
      first = data[0];}
 else{first = 0;}

 if(nrhs > 3)
     {data = mxGetPr(prhs[3]);
      nFrames = data[0];}
 else{nFrames = 1;}

    /*----------- open file --------------------------*/

  status=mxGetString(prhs[0],fileName,100);
  status=mxGetString(prhs[1],adcName,100);
  mexPrintf(" Open file:%s for channel:%s\n",fileName,adcName);
  mexPrintf(" first frame:%d nFrames=%d\n",first,nFrames);

  buffSize = 300000;
  buff = mxCalloc(buffSize,1);
  iFile = FrFileINew(fileName,buff,buffSize);
  if(iFile == NULL)
     {printf("Error during file opening\n"
             "  Last errors are:\n%s",FrErrorGetHistory());
      return;}

     /*--------- read the first frame ------------------*/
   
  while((frame = FrameRead(iFile)) != NULL)
      {
	mexPrintf(" Frame %d run %d ",frame->frame, frame->run);
	mexPrintf(" Starting GPS time:%s (and %d usec)\n",
	       FrStrGTime(frame->GTimeS),frame->GTimeN/1000);
       if(first == 0) break;
       if(frame->frame >= first) break; 
       FrameFree(frame);}

  if(frame == NULL) return;

     /*------ find ADC and create output matrix ---------*/

  adc = FrAdcDataFind(frame,adcName);
  if(adc == NULL) return;

  nBin = adc->data->nData;
  nData = nBin*nFrames;
  dt = 1./adc->sampleRate;
  df = adc->sampleRate/((double)nData);
  plhs[0] = mxCreateDoubleMatrix(1,nData,mxREAL);
 
    /*------ fill time and frequency array ---------------*/

  if(nlhs > 1)
    {plhs[1] = mxCreateDoubleMatrix(1,nData,mxREAL);
     data = mxGetPr(plhs[1]);
     for(i=0; i<nData; i++)
        {data[i] = (double)i*dt;}}

  if(nlhs > 2)
    {plhs[2] = mxCreateDoubleMatrix(1,nData/2,mxREAL);
     data = mxGetPr(plhs[2]);
     for(i=0; i<nData/2; i++)
        {data[i] = (double)i*df;}}

  if(nlhs > 3)
    {plhs[3] = mxCreateDoubleMatrix(1,1,mxREAL);
     data = mxGetPr(plhs[3]);
     data[0] = (double)frame->GTimeS + 1.e-9*(double)frame->GTimeN;}

  if(nlhs > 4)
    {tp = frame->GTimeS + 315961200;
     strncpy(stringTime,ctime(&tp),24);
     name = mxCalloc(80,sizeof(char));
     sprintf(name,"Starting GPS time:%s (and %d usec)\n",
                            stringTime,frame->GTimeN/1000);
     plhs[4] = mxCreateString(name);
     mxFree(name);}	

  if(nlhs > 5)
    {if(adc->comment != NULL)
         {name = mxCalloc(strlen(adc->comment)+1,sizeof(char));
          strcpy(name, adc->comment);
          plhs[5] = mxCreateString(name);
          mxFree(name);}		
     else{plhs[5] = mxCreateString("no comment");}}

  if(nlhs > 6)
    {if(adc->units != NULL)
         {name = mxCalloc(strlen(adc->units)+1,sizeof(char));
          strcpy(name, adc->units);
          plhs[6] = mxCreateString(name);
          mxFree(name);}
     else{plhs[6] = mxCreateString("no unit");}}

  if(nlhs > 7)
    {plhs[7] = mxCreateDoubleMatrix(1,9,mxREAL);
     data = mxGetPr(plhs[7]);
     data[0] = (double) adc->crate;
     data[1] = (double) adc->channel;
     data[2] = (double) adc->nBits;
     data[3] = (double) adc->bias;
     data[4] = (double) adc->slope;
     data[5] = (double) adc->sampleRate;
     data[6] = (double) adc->timeOffsetS + .1e-9 * (double)adc->timeOffsetN;
     data[7] = (double) adc->fShift;
     data[8] = (double) adc->overRange;}


    /*-------- copy data and read the next frames -------*/

  data = mxGetPr(plhs[0]);
  index = 0;
  while(adc != NULL)
    {vect = adc->data;
     if(vect == NULL) 
        {printf(" no data for frame %ld\n",frame->frame);}
     else if(vect->type == FR_VECT_C)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->data[i];}}
     else if(vect->type == FR_VECT_2S)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataS[i];}}
     else if(vect->type == FR_VECT_4S)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataI[i];}}
     else if(vect->type == FR_VECT_1U)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataU[i];}}
     else if(vect->type == FR_VECT_2U)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataUS[i];}}
     else if(vect->type == FR_VECT_4U)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataUI[i];}}
     else if(vect->type == FR_VECT_4R)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataF[i];}}
     else if(vect->type == FR_VECT_8R)
        {for(i=0; i<nBin; i++) {data[i+index] = vect->dataD[i];}}
     index += nBin;

     FrameFree(frame);
     frame = FrameRead(iFile);
     if(frame == NULL) break; 
     if(index >= nFrames*nBin) break; 

     adc = FrAdcDataFind(frame,adcName);}

  
    
  FrFileIEnd(iFile);            /* Close file and release descriptor */
  mxFree(buff);                 /* Free tempory buffer */

}
