/*--------------------------------------------------------------------*/
/* File: FrCopy.c  v3.74                                 D. Verkindt  */
/*                                          Last update: 10 Feb. 1999 */
/*                                                                    */
/*  This program copy frames from input files                         */
/*  to one single output file, using or not data compression.         */
/*  Giving an input directory will write each input file of the       */
/*  directory in one single output file. Giving no output file will   */
/*  make FrCopy write in <inputName>.dat or <inputName>.copy          */
/*  depending on the input (directory or file).                       */
/*  Frames written are by default compressed.                         */
/*      Synthax is: FrCopy -i <input file or directory>               */
/*                         -o <ouput file>                            */
/*                         -r <begin run number> <end run number>     */
/*                         -f <begin frame number> <end frame number> */
/*                         -c <compression type>                      */
/*                         -l <level of compression>                  */
/*                         -debug <debug level>                       */
/*                         -h (to get help)                           */
/*--------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include "FrameL.h"

int debug = 0;

#define LBUFF_IN    540000
#define LSTRING_MAX 256
#define NFILE_MAX    100000
#define DEFAULT1    -1
#define DEFAULT2     10000000


struct Parameters{
  char *inName;       /* input file or directory to copy  */
  char *outName;      /* output file of the copy          */
  long run1;          /* copy from run1 to run2           */
  long run2;          /* copy from run1 to run2           */
  long frame1;        /* copy from frame1 to frame2       */
  long frame2;        /* copy from frame1 to frame2       */
  long ctype;         /* type of compression              */
  long level;         /* level of compression             */
  long nFrame;        /* number of frames copied          */
  long nFile;         /* number of files read for copy    */
};


void warning(int arg);
void SetParameters(int argc, char **argv, struct Parameters *par);
long SetSortedFileList(char *indirName, char **fileName);
char *my_strcpy(char *dummy, char *str);
long is_a_dir(const char *str);


struct Parameters *par;


/*-------------------------------------------------------- main ------------*/

int main(int argc, char **argv)
{long   i, cpt, framecpt;
 char   *buffO, *buffI;
 struct FrFile *oFile, *iFile;
 struct FrameH *frame;
 char   **fileName;



 /*----- Deal with input arguments -----*/
 par = (struct Parameters *)calloc(1,sizeof(struct Parameters));
 SetParameters(argc, argv, par);

 if (debug>1)
    {printf("\n Arguments used:\n");
     printf("   inName       = %s \n",           par->inName);
     printf("   outame       = %s \n",           par->outName);
     printf("   runNumbers   = %ld  to  %ld \n", par->run1, par->run2);
     printf("   frameNumbers = %ld  to  %ld \n", par->frame1, par->frame2);
     printf("   ctype        = %ld \n",          par->ctype);
     printf("   level        = %ld \n",          par->level);
     sleep(1);
    }


 /*----- Get and Sort the files containing the frames to be copied -----*/

 fileName   = (char **) calloc(NFILE_MAX, sizeof(char *));

 if (!is_a_dir(par->inName))
    {par->nFile  = 1;
     fileName[0] = my_strcpy(fileName[0], par->inName);
    }
 else
    {par->nFile = SetSortedFileList(par->inName, fileName);}

 if (par->nFile <= 0) warning(0);
 if (debug>1) printf("   nFile        = %ld \n\n", par->nFile);


 /*----- open the output file in binary format with compression ctype -----*/

 buffO     = malloc(LBUFF_IN);
 oFile     = FrFileONew(par->outName, par->ctype, buffO, LBUFF_IN); 
 if(oFile == NULL)
    {printf("Error during output file creation\n"
            "  Last errors are:\n%s",FrErrorGetHistory());
     warning(0);
    }

 /*---- Set level of frames compression in output file -----*/
 FrFileOSetGzipLevel(oFile, par->level);




 /*----- Main loop ------*/

 buffI = malloc(LBUFF_IN);
 if (buffI == NULL)
     {printf("buffI not allocated. Try again.\n");
      return(0);
     }

 cpt = 0;
 for (i=0; i<par->nFile; i++)
    {
     if (debug>10) printf("fileName[%ld] = %s \n",i,fileName[i]);

     iFile     = FrFileINew (fileName[i], buffI, LBUFF_IN);
     if(iFile == NULL)
        {printf("Error during opening of file %ld (%s)\n"
                "  Last errors are:\n%s",i,fileName[i],FrErrorGetHistory());
         warning(1);
         continue;
        }
      if (debug>0)
         {printf("\n\n-----------------------------------------------------------------------\n");
          printf("   I copy frames between (run %ld frame %ld) and (run %ld frame %ld) \n",
                     par->run1, par->frame1, par->run2, par->frame2);
          printf("   from file %s \n   to file %s \n", fileName[i], par->outName);
          printf("   (compress_type=%ld , gzip_lvl=%ld) \n", par->ctype, par->level);
          printf("--------------------------------------------------------------------------\n\n");
         }

     /*----- read frames and write it to output file (with compression ctype) -----*/
     framecpt = 0;
     while((frame = FrameRead(iFile)) != NULL)
        {if ((frame->run == par->run2 || par->run2 == DEFAULT2) && frame->frame > par->frame2)
                {FrameFree(frame);
                 i = par->nFile;
                 break;
                }
         if (frame->run < par->run1 || frame->run > par->run2 ||
             frame->frame < par->frame1 || frame->frame > par->frame2)
                {FrameFree(frame);
                 continue;
                }
         framecpt++;
         cpt++;
         if (debug>1)
            {printf("    copy frame %d of run %d \n",frame->frame, frame->run);}
         if(FrameWrite(frame, oFile) != FR_OK)
             {printf("Error during frame write\n"
                     "  Last errors are:\n%s",FrErrorGetHistory());
              exit(0);
             }
         FrameFree(frame);
        }
     FrFileIEnd(iFile);
    }


 /*----- Close output and free allocated memory -----*/

 FrFileOEnd(oFile);
 free(buffI);
 free(buffO);
 if (cpt == 0) printf("\n\n Nothing done. Run or Frame numbers are out of range.\n");
 else          printf("\n\n %ld Frames copied to file %s \n", cpt, par->outName);
 printf("\n -- OK. FrCopy Done.\n\n");
 exit(0);
}





/*------------------------------------------------------------------- warning ------*/

void warning(int arg)
{
 if (arg==1)
   {printf("\n---------------------------------------------------------------------\n");
    printf("* Warning: file has not been read. May be, it does not contain frames! *\n");
    printf("-----------------------------------------------------------------------\n\n");
   }
 if (arg==0)
   {printf("\n\n--------------------------------------------------------------------------\n");
    printf("          FrCopy               \n");   
    printf("      A problem occured        \n");   
    printf("                               \n");
    printf("      ! W A R N I N G !        \n");
    printf("                               \n");
    printf("       type FrCopy -h          \n"); 
    printf("        to get help            \n");
    printf("                               \n"); 
    printf("--------------------------------------------------------------------------\n\n");
    exit(0);
   }
 return;
}




/*-------------------------------------------------------------- SetParameters ------*/

void SetParameters(int argc, char **argv, struct Parameters *par)
{int i;

 if (debug>10) printf("\n SetParameters \n");

 /*-------- default values ---------*/
 par->inName  = NULL;
 par->outName = NULL;
 par->run1    = DEFAULT1;
 par->frame1  = DEFAULT1;
 par->run2    = DEFAULT2;
 par->frame2  = DEFAULT2;
 par->ctype   = 3;
 par->level   = 1;

 /*-------- online help --------*/
 if (argc==1 || (argc>1 && strncmp(argv[1],"-h",2) == 0))
    {printf(" \n");
     printf(" This program copy frames from input files                           \n");
     printf(" to one single output file, using or not data compression.           \n");
     printf(" Giving an input directory will write each input file of the         \n");
     printf(" directory in one single output file. Giving no output file will     \n");
     printf(" make FrCopy write in <inputName>.dat or <inputName>.copy            \n");
     printf(" depending on the input (directory or file).                         \n");
     printf(" Frames written are by default compressed.                           \n");
     printf("-------------------------------------------------------------------- \n");
     printf(" Syntax is: FrCopy  -i <input file or directory>                     \n");
     printf("                    -r <begin run number> <end run number>           \n"); 
     printf("                    -f <begin frame number> <end framenumber>        \n");
     printf("                    -c <compression type>                            \n");
     printf("                    -l <level of dump>                               \n");
     printf("                    -debug <debug level>                             \n");
     printf("                    -h (to get help)                                 \n");
     printf("-------------------------------------------------------------------- \n\n");
     printf("  -i : Argument can be an input file or an input directory where are \n");
     printf("       the files to copy (compress) into a single output file.       \n");
     printf("  -o : The output file is located by default in the same directory   \n");
     printf("       as the input file or input directory.                         \n");
     printf("  -r : Giving one run number is like giving twice the same           \n");
     printf("       Giving two run number will extract the runs in between.       \n");
     printf("  -f : Giving one frame number is like giving twice the same         \n");
     printf("       Giving two frame number will extract the frames in between.   \n");
     printf("  -c : Compression type is 0 (no compression), 1 (gzip compression)  \n");
     printf("       2 (differenciation) (does not compress) or 3 (differenciation+gzip) \n");
     printf("       (default is 3). To decompress a file, copy it using 0.        \n");
     printf("  -l : Compression level is 0 (no compression), 1-9 (gzip level).    \n");
     printf("       In most of the case, default level=1 is enough.               \n");
     printf("-------------------------------------------------------------------- \n");
     printf(" \n");
     exit(0);
    }

 if (argc == 2)
      {par->inName = my_strcpy(par->inName, argv[1]); argc = 1;}

 for (i=1; i<argc-1; i++)
    {if (strcmp(argv[i],"-i") == 0)
        {par->inName = my_strcpy(par->inName, argv[i+1]);
         i++;
        }
     if (strcmp(argv[i],"-o") == 0)
        {par->outName = my_strcpy(par->outName, argv[i+1]);
         i++;
        }
     if (strcmp(argv[i],"-r") == 0)
        {par->run1 = atoi(argv[i+1]);
         if (i+2 == argc)
            {par->run2 = par->run1;
             i++;
             break;
            }
         if (atoi(argv[i+2]) != 0)
            {par->run2 = atoi(argv[i+2]);
             i = i + 2;
            }
         else
            {par->run2 = par->run1;
             i++;
            }
        }
     if (strcmp(argv[i],"-f") == 0)
        {par->frame1 = atoi(argv[i+1]);
         if (i+2 == argc)
            {par->frame2 = par->frame1;
             i++;
             break;
            }
         if (atoi(argv[i+2]) != 0)
            {par->frame2 = atoi(argv[i+2]);
             i = i + 2;
            }
         else
            {par->frame2 = par->frame1;
             i++;
            }
        }
     if (strcmp(argv[i],"-c") == 0)
        {par->ctype = atoi(argv[i+1]);
         i++;
        }
     if (strcmp(argv[i],"-l") == 0)
        {par->level = atoi(argv[i+1]);
         i++;
        }
     if (strcmp(argv[i],"-debug") == 0)
        {debug = atoi(argv[i+1]);
         i++;
        }
    }

 if (!par->inName)
    {printf("\n  You must give me at least a file or a directory name (with -i)\n\n");
     exit(0);
    }
 if (!par->outName && is_a_dir(par->inName))
    {par->outName = (char*)calloc(strlen(par->inName)+4,sizeof(char));
     sprintf(par->outName,"%s.dat",par->inName);
    }
 if (!par->outName && !is_a_dir(par->inName))
    {par->outName = (char*)calloc(strlen(par->inName)+5,sizeof(char));
     sprintf(par->outName,"%s_copy",par->inName);
    }
 if (par->ctype < 0 || par->ctype > 3) par->ctype = 3;
 if (par->level < 0)                   par->level = 1;
 if (par->level > 9)                   par->level = 9;

 if (par->ctype == 0 || par->ctype == 2) par->level = 0;

 return;
}





/*------------------------------------------------------ SetSortedFileList ------*/

long SetSortedFileList(char *dirName, char **fileName)
{long nfile, i, j;
 struct FrFile *iFile;
 struct FrameH *frame;
 char *buffI;
 char inName[LSTRING_MAX],infileName[LSTRING_MAX],dummy[LSTRING_MAX];
 long gtimeS_frame1[NFILE_MAX], gtimeN_frame1[NFILE_MAX], ffof;
 DIR  *dirp;
 struct dirent *entry;

 if (debug>10) printf("\n SetSortedFileList \n");

 if (debug>10) printf(" SetSortedFileList: I get the list of files to be sorted, be patient... \n");

 /* We get the first frame of all the files of the list */

 buffI = malloc(LBUFF_IN);
 if (buffI == NULL)
     {printf("buffI not allocated. Try again.\n");
      return(0);
     }
 nfile = 0;

 dirp = opendir(dirName);
 while((entry = readdir(dirp)) != NULL)
   {
   if (strncmp(entry->d_name,".",1) == 0) continue;

    strcpy(infileName,entry->d_name);
    if (is_a_dir(dirName)) sprintf(inName,"%s/%s",dirName,infileName);
    else                   sprintf(inName,"%s",infileName);

    if (debug>20)
       {printf("  inName=%s \n infileName=%s \n dirName=%s \n",
               inName, infileName, dirName);}

    iFile     = FrFileINew (inName, buffI, LBUFF_IN);
    if(iFile == NULL)
        {printf(" SetSortedFileList: Error during file opening\n");
         continue;
        }

    frame = FrameRead(iFile);

    if (frame == NULL)
        {printf("In sorted_files_of_dir: Error during frame read\n");
         FrFileIEnd(iFile);
         continue;
        }
    if (frame->GTimeS >0) gtimeS_frame1[nfile] = frame->GTimeS;
    else                  gtimeS_frame1[nfile] = frame->run * 100000 + frame->frame;
    if (frame->GTimeN >0) gtimeN_frame1[nfile] = frame->GTimeN;

    fileName[nfile] = my_strcpy(fileName[nfile], inName);

    if (debug>10) printf("    DySortFiles: fileName[%ld]=%s \n\n",nfile,fileName[nfile]);
    FrameFree(frame);
    FrFileIEnd(iFile);
    nfile++;
   }

 closedir(dirp);
 free(buffI);

 if (nfile<1) {printf(" SetSortedFileList: No files read. \n"); return(0);}

 if (debug>10) printf(" SetSortedFileList: Now I sort the file list \n");
 for (i=0;i<nfile-1;i++)
  {for (j=i+1;j<nfile;j++)
     {if ((gtimeS_frame1[i] > gtimeS_frame1[j]) ||
          (gtimeS_frame1[i] == gtimeS_frame1[j] && gtimeN_frame1[i] > gtimeN_frame1[j]))
              {strcpy(dummy,fileName[i]);       ffof = gtimeS_frame1[i];
               strcpy(fileName[i],fileName[j]); gtimeS_frame1[i] = gtimeS_frame1[j];
               strcpy(fileName[j],dummy);       gtimeS_frame1[j] = ffof;
              }
     }
  }
 return(nfile);
}




/*---------------------------------------------------- my_strcpy ---------*/
char *my_strcpy(char *dummy, char *str)
{
 if (debug>200) printf("\nmy_strcpy \n");

 if (!str) return(NULL);

 if (dummy) free(dummy);
 dummy = (char *) calloc(strlen(str)+1,sizeof(char));
 strcpy(dummy, str);

 return(dummy);
}



/*----------------------------------------------------- Is_a_dir ----------*/

long is_a_dir(const char *str)
{DIR *fd;

 fd = opendir(str);
 if (fd)
   {closedir(fd);
    return 1;
   }
 return 0;
}
