/*-------------------------------------------------------------------------*/
/* File: FrDump.c   v3.74                                     D. Verkindt  */
/*                                               Last update: 10 Feb. 1999 */
/*                                                                         */
/*  This program produces a dump of one file, one run, one frame           */
/*  or one data type (raw,adc,sms,proc,trig,...).                          */
/*      Synthax is: FrDump -i <input file or directory>                    */
/*                         -r <begin run number> <end run number>          */
/*                         -f <begin frame number> <end frame number>      */
/*                         -var <variable name or adc,sms,raw,proc,trig>   */
/*                         -l <level of dump>                              */
/*                         -debug <debug level>                            */
/*                         -h (to get help)                                */
/*-------------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include "FrameL.h"

int debug = 0;


#define LBUFF_IN    540000
#define LSTRING_MAX 256
#define NFILE_MAX    100000
#define DEFAULT1     -1
#define DEFAULT2     10000000

struct Parameters{
  char *inName;       /* input file to dump            */
  long run1;          /* dump from run1 to run2        */
  long run2;          /* dump from run1 to run2        */
  long frame1;        /* dump from frame1 to frame2    */
  long frame2;        /* dump from frame1 to frame2    */
  char *varName[20];  /* list of variables to dump     */
  int  nvar;          /* number of variables to dump   */
  long dumpLvl;       /* level of frame's dump         */
  long nFrame;        /* number of frames dumped       */
  long nFile;         /* number of files read for dump */
};

void warning(int arg);
void SetParameters(int argc, char **argv, struct Parameters *par);
long SetSortedFileList(char *inName, char **fileName);

long DumpFileInfo(struct FrFile *iFile, char *fileName);
void DumpOneVar(struct FrameH *frame, char *varName, long level);
void DumpAllAdc(struct FrameH *frame, long level);
void DumpAllSer(struct FrameH *frame, long level);
void DumpAllProc(struct FrameH *frame, long level);
void DumpAllTrig(struct FrameH *frame, long level);
void DumpAllSim(struct FrameH *frame, long level);
void DumpAllStatic(struct FrameH *frame, long level);

char *my_strcpy(char *dummy, char *str);
long is_a_dir(const char *str);


struct Parameters *par;


/*------------------------------------------------------- main ----*/

int main(int argc, char **argv)   
{
 struct FrFile *iFile;
 struct FrameH *frame;
 long   i, j, cpt;
 char   *buffI;
 char   **fileName;


 /*----- Get input arguments -----*/
 par = (struct Parameters *)calloc(1,sizeof(struct Parameters));
 SetParameters(argc, argv, par);

 /*----- Verify input arguments -----*/
 if (debug>1)
    {printf("\n Arguments used:\n"); 
     printf("   inName    = %s \n",              par->inName);
     printf("   runNumbers   = %ld  to  %ld \n", par->run1, par->run2);
     printf("   frameNumbers = %ld  to  %ld \n", par->frame1, par->frame2);
     printf("   nvar          = %d \n",          par->nvar);
     for (i=0; i<par->nvar; i++) printf("      var %ld = %s \n",i, par->varName[i]);
     printf("   dumpLvl      = %ld \n\n",        par->dumpLvl);
     sleep(1);
    }


 /*----- Get and Sort the files containing the frames to be copied -----*/
 fileName = (char **) calloc(NFILE_MAX, sizeof(char *));
 if (is_a_dir(par->inName))
    {par->nFile = SetSortedFileList(par->inName, fileName);}
 else
    {par->nFile  = 1;
     fileName[0] = my_strcpy(fileName[0], par->inName);
    }
 if (par->nFile <= 0) warning(0);


 if (debug>100)
     {printf("=== nFile=%ld \n",par->nFile);
      for (i=0; i<par->nFile; i++) printf("==== fileName[%ld]=%s \n",i,fileName[i]);
     }


 /*------------------------------ Main loop -----------------------------*/

 if (debug>1) printf("\n Open input files and Dump the selected frames \n\n");

 buffI = malloc(LBUFF_IN);
 if (buffI == NULL)
     {printf("buffI not allocated. Try again.\n");
      return(0);
     }

 cpt = 0;
 for (i=0; i<par->nFile; i++)
   {
    if (debug>10) printf("fileName[%ld] = %s \n",i,fileName[i]);

    iFile     = FrFileINew (fileName[i], buffI, LBUFF_IN);
    if(iFile == NULL)
       {printf("Error during file opening\n"
               "  Last errors are:\n%s",FrErrorGetHistory());
        warning(0);
       }


    /*-------- Dump frame's info of one file --------*/

    if (par->inName && par->nvar == 0 
        && par->run1 == DEFAULT1 && par->frame1 == DEFAULT1
        && par->run2 == DEFAULT2 && par->frame2 == DEFAULT2 )
                {par->nFrame = DumpFileInfo(iFile, fileName[i]);
                 cpt += par->nFrame;
                 continue;
                }

    /*-------- Look at frames of file i --------*/

    while((frame = FrameRead(iFile)) != NULL)
       {if ((frame->run == par->run2 || par->run2 == DEFAULT2) && frame->frame > par->frame2)
               {FrameFree(frame);
                i = par->nFile;
                break;
               }
        if (frame->run < par->run1 || frame->run > par->run2 ||
            frame->frame < par->frame1 || frame->frame > par->frame2)
               {FrameFree(frame);
                continue;
               }

        printf("\n --------------------------------------------------------------------- \n");
        printf("  %s   run %i  frame %i  on %s \n",
                  fileName[i], frame->run, frame->frame, FrStrGTime(frame->GTimeS));
        printf(" --------------------------------------------------------------------- \n");


        /*-------- dump one frame ----------*/
        if (par->inName && par->nvar == 0)
                {FrameDump(frame, stdout, par->dumpLvl);}

        /*-------- dump variables in the frame ----------*/
        if (par->inName && par->nvar > 0)
          {
           for (j=0; j<par->nvar; j++)
              {
               /*--- dump raw data (adc+sms) of one frame ---*/
               if (strcmp(par->varName[j],"raw")==0)
                      {if(frame->rawData == NULL)
                          {printf("    No raw data in this frame. \n");}
                       else
                          {FrRawDataDump(frame->rawData, stdout, par->dumpLvl);}
                      }  

               /*--- dump one variable of one frame ---*/
               if (strcmp(par->varName[j],"adc") != 0  && strcmp(par->varName[j],"sms") != 0  &&
                   strcmp(par->varName[j],"raw") != 0  && strcmp(par->varName[j],"proc") != 0 && 
                   strcmp(par->varName[j],"trig") != 0 && strcmp(par->varName[j],"sim") != 0  && 
                   strcmp(par->varName[j],"static") != 0 )
                      {DumpOneVar(frame, par->varName[j], par->dumpLvl);}

               /*--- dump all adc data of one frame ---*/
               if (strcmp(par->varName[j],"adc") == 0)
                      {DumpAllAdc(frame, par->dumpLvl);}
 
               /*--- dump slow monitoring data of one frame ---*/
               if (strcmp(par->varName[j],"sms") == 0)
                      {DumpAllSer(frame, par->dumpLvl);}

               /*--- dump all proc data of one frame ---*/
               if (strcmp(par->varName[j],"proc") == 0)
                      {DumpAllProc(frame, par->dumpLvl);}
 
               /*--- dump all trigger data of one frame ---*/
               if (strcmp(par->varName[j],"trig") == 0)
                      {DumpAllTrig(frame, par->dumpLvl);}

               /*--- dump all sim data of one frame ---*/
               if (strcmp(par->varName[j],"sim") == 0)
                      {DumpAllSim(frame, par->dumpLvl);}

               /*--- dump all static data of one frame ---*/
               if (strcmp(par->varName[j],"static") == 0)
                      {DumpAllStatic(frame, par->dumpLvl);}
              }
           }
        cpt++;
        FrameFree(frame);
       }

    FrFileIEnd(iFile);
   }


 /*------------- ending --------------*/
 free(buffI);
 if (cpt == 0) printf("\n\n Nothing done. Run or Frame numbers are out of range.\n");
 else          printf("\n\n %ld frames read with FrDump. \n",cpt);
 printf("\n -- OK. FrDump Done.\n\n");
 exit(0);
}







/*------------------------------------------------------------------- warning ------*/

void warning(int arg)
{
 if (arg==1)
   {printf("\n---------------------------------------------------------------------\n");
    printf("* Warning: file has not been read. May be, it does not contain frames! *\n");
    printf("-----------------------------------------------------------------------\n\n");
   }

 if (arg==0)
   {printf("\n\n-------------------------------\n");
    printf("          FrDump               \n");   
    printf("      A problem occured        \n");   
    printf("                               \n");
    printf("      ! W A R N I N G !        \n");
    printf("                               \n");
    printf("       type FrDump -h          \n"); 
    printf("        to get help            \n");
    printf("                               \n"); 
    printf("-------------------------------\n\n");
    exit(0);
   }

 return;
}




/*-------------------------------------------------------------- SetParameters ------*/

void SetParameters(int argc, char **argv, struct Parameters *par)
{int i;

 if (debug>10) printf("\n SetParameters \n");

 /*-------- default values ---------*/
 par->inName  = NULL;
 par->run1    = DEFAULT1;
 par->frame1  = DEFAULT1;
 par->run2    = DEFAULT2;
 par->frame2  = DEFAULT2;
 par->nvar    = 0;
 par->dumpLvl = 2;

 /*-------- online help --------*/
 if (argc==1 || (argc>1 && strncmp(argv[1],"-h",2) == 0))
    {printf(" \n");
     printf("------------------------------------------------------------------------------ \n");
     printf("  Syntax is: FrDump  -i     <input file or directory>                          \n");
     printf("                     -var   <variable name or adc,sms,raw,proc,trig,sim,static>\n");
     printf("                     -r     <begin run number> <end run number>                \n"); 
     printf("                     -f     <begin frame number> <end framenumber>             \n");
     printf("                     -l     <level of dump>                                    \n");
     printf("                     -debug <debug level>                                      \n");
     printf("                     -h     (to get help)                                      \n");
     printf("------------------------------------------------------------------------------ \n");
     printf("  -i   : Argument can be an input file or an input directory where are         \n");
     printf("         the files to copy (compress) into a single output file.               \n");
     printf("  -var : each variable you want to dump. If adc,sms,raw,proc,trig,sim or static\n");
     printf("         then all the adc,sms,raw,proctrig,sim or static variables are dumped. \n");
     printf("  -r   : Giving one run number is like giving twice the same                   \n");
     printf("  -f   : Giving one frame number is like giving twice the same                 \n");
     printf("  -l   : default dump level (2) gives quite all the most relevant informations.\n");
     printf("------------------------------------------------------------------------------ \n");
     printf(" \n");
     exit(0);
    }

 if (argc == 2)
     {par->inName = my_strcpy(par->inName, argv[1]); argc = 1;}

 for (i=1; i<argc-1; i++)
    {
     if (strcmp(argv[i],"-i") == 0)
        {par->inName = my_strcpy(par->inName, argv[i+1]);
         i++;
        }
     if (strcmp(argv[i],"-var") == 0)
        {par->varName[par->nvar] = my_strcpy(par->varName[par->nvar], argv[i+1]);
         i++;
         par->nvar++;
        }
     if (strcmp(argv[i],"-r") == 0)
        {par->run1 = atoi(argv[i+1]);
         if (i+2 == argc)
             {par->run2 = par->run1;
              i++;
              break;
             }
         if (atoi(argv[i+2]) != 0)
             {par->run2 = atoi(argv[i+2]);
              i = i + 2;
             }
         else
             {par->run2 = par->run1;
              i++;
             }
        }
     if (strcmp(argv[i],"-f") == 0)
        {par->frame1 = atoi(argv[i+1]);
         if (i+2 == argc)
             {par->frame2 = par->frame1;
              i++;
              break;
             }
         if (atoi(argv[i+2]) != 0)
             {par->frame2 = atoi(argv[i+2]);
              i = i + 2;
             }
         else
             {par->frame2 = par->frame1;
              i++;
             }
        }
     if (strcmp(argv[i],"-l") == 0)
        {par->dumpLvl = atoi(argv[i+1]);
          i++;
        }
     if (strcmp(argv[i],"-debug") == 0)
        {debug = atoi(argv[i+1]);
         i++;
        }
    }

 if (!par->inName)
    {printf("\n  You must give me at least a file or a directory name (option -i)\n\n");
     exit(0);}

 return;
}




/*------------------------------------------------------ SetSortedFileList ------*/

long SetSortedFileList(char *dirName, char **fileName)
{long nfile, i, j;
 struct FrFile *iFile;
 struct FrameH *frame;
 char *buffI;
 char inName[LSTRING_MAX],infileName[LSTRING_MAX],dummy[LSTRING_MAX];
 long gtimeS_frame1[NFILE_MAX], gtimeN_frame1[NFILE_MAX], gts,gtn;
 DIR  *dirp;
 struct dirent *entry;

 if (debug>10) printf("\n SetSortedFileList \n");

 if (debug>10) printf(" SetSortedFileList: I get the list of files to be sorted, be patient... \n");

 /* We get the first frame of all the files of the list */

 buffI = malloc(LBUFF_IN);
 if (buffI == NULL)
     {printf("buffI not allocated. Try again.\n");
      return(0);
     }
 nfile = 0;

 dirp = opendir(dirName);
 while((entry = readdir(dirp)) != NULL)
   {
    if (strncmp(entry->d_name,".",1) == 0) continue;

    strcpy(infileName,entry->d_name);
    if (is_a_dir(dirName)) sprintf(inName,"%s/%s",dirName,infileName);
    else                   sprintf(inName,"%s",infileName);

    if (debug>20)
       {printf("  inName=%s \n infileName=%s \n dirName=%s \n",
               inName, infileName, dirName);}

    iFile     = FrFileINew (inName, buffI, LBUFF_IN);
    if(iFile == NULL)
         {printf(" SetSortedFileList: Error during file opening\n");
          continue;
         }

    frame = FrameRead(iFile);

    if (frame == NULL)
         {printf("In sorted_files_of_dir: Error during frame read\n");
          FrFileIEnd(iFile);
          continue;
         }

    if (frame->GTimeS >0) gtimeS_frame1[nfile] = frame->GTimeS;
    else                  gtimeS_frame1[nfile] = frame->run * 100000 + frame->frame;
    if (frame->GTimeN >0) gtimeN_frame1[nfile] = frame->GTimeN;

    fileName[nfile] = my_strcpy(fileName[nfile], inName);

    FrameFree(frame);
    FrFileIEnd(iFile);
    nfile++;
   }

 closedir(dirp);
 free(buffI);

 if (nfile<1)
     {printf(" SetSortedFileList: No files read. \n");
      return(0);
     }

 if (debug>10) printf("  Now I sort the file list... \n");

 for (i=0;i<nfile-1;i++)
  {for (j=i+1;j<nfile;j++)
     {if ((gtimeS_frame1[i] > gtimeS_frame1[j]) ||
          (gtimeS_frame1[i] == gtimeS_frame1[j] && gtimeN_frame1[i] > gtimeN_frame1[j]))
              {strcpy(dummy,fileName[i]);
               gts = gtimeS_frame1[i];
               gtn = gtimeN_frame1[i];
               strcpy(fileName[i],fileName[j]);
               gtimeS_frame1[i] = gtimeS_frame1[j];
               gtimeN_frame1[i] = gtimeN_frame1[j];
               strcpy(fileName[j],dummy);
               gtimeS_frame1[j] = gts;
               gtimeN_frame1[j] = gtn;
              }
     }
  }

 return(nfile);
}




/*----------------------------------------------------- DumpFileInfo ----------*/

long DumpFileInfo(struct FrFile *iFile, char *fileName)
{struct FrameH *frame;
 static long timeSa=0, timeNa=0;
 long nframe,runmin,runmax,framemin,framemax;
 long timeSmin,timeSmax,timeS,timeNmin,timeNmax,timeN;
 long size;
 FILE *fp;

 if (debug>10) printf("\n DumpFileInfo \n");

 fp = fopen(fileName,"r");
 fseek(fp,0,SEEK_END);
 size  = ftell(fp);

 nframe = 0;
 while((frame = FrameRead(iFile)) != NULL)
   {timeS = frame->GTimeS;
    timeN = frame->GTimeN;

    if (timeS > 0 && (timeS>timeSa || (timeS==timeSa && timeN>timeNa)))
      {if (nframe==0 || timeSmin > timeS || (timeSmin==timeS && timeNmin > timeN))
             {timeSmin = timeS;
              timeNmin = timeN;
              runmin   = frame->run;
              framemin = frame->frame;
             }
       if (nframe==0 || timeSmax < timeS || (timeSmax==timeS && timeNmax < timeN))
             {timeSmax = timeS;
              timeNmax = timeN;
              runmax   = frame->run;
              framemax = frame->frame;
             }
       timeSa = timeS;
       timeNa = timeN;
      }
    else
      {if (nframe==0 || framemin > frame->frame)
             {timeSmin = timeS;
              runmin   = frame->run;
              framemin = frame->frame;
             }
       if (nframe==0 || framemax < frame->frame)
             {timeSmax = timeS;
              runmax   = frame->run;
              framemax = frame->frame;
             }
      }
    nframe++;
    FrameFree(frame);
   }

 printf("\n ---------------------------------------------------------------------------- \n");
 printf(" File %s (%.2fkB) contains %ld Frames \n", fileName, size/1000., nframe);
 if (timeS > 0)
    {printf("   earliest: run %6ld  frame %6ld   on %s \n",runmin,framemin,FrStrGTime(timeSmin));
     printf("   lastest:  run %6ld  frame %6ld   on %s \n",runmax,framemax,FrStrGTime(timeSmax));
    }
 else
    {printf("   runmin = %6ld  framemin = %6ld  (no time information available) \n",runmin,framemin);
     printf("   runmax = %6ld  framemax = %6ld  (no time information available) \n",runmax,framemax);
    }
 printf(" ---------------------------------------------------------------------------- \n");

 return(nframe);
}





/*----------------------------------------------------- DumpOneVar ----------*/

void DumpOneVar(struct FrameH *frame, char *varName, long level)
{struct FrAdcData  *adcData;
 struct FrSerData  *serData;
 struct FrProcData *procData;
 struct FrTrigData *trigger;
 struct FrSimData  *simData;
 long flag;

 if (debug>10) printf("\n DumpOneVar \n");
 flag = 0;

 if (!frame) return;

 if (frame->rawData)
   {
    adcData = FrAdcDataFind(frame, varName);
    if (adcData != NULL)
      {printf(" ADC: %s \n",adcData->name);
       printf("     nBits=%i  sampleRate=%5.1f  crate=%i  channel=%i \n",
                    adcData->nBits, adcData->sampleRate, adcData->crate, adcData->channel);
       if (adcData->data != NULL) FrVectDump(adcData->data, stdout, level);
       flag = 1;
      }

    serData = frame->rawData->firstSer;
    while (serData != NULL)
       {if (strcmp(serData->name,varName)==0) break;
        serData = serData->next;
       }

    if (serData != NULL)
      {printf(" SMS: %s \n",serData->name);
       printf("     timeSec=%s  timeNsec=%i  sampleRate=%5.1f  \n",
                    FrStrGTime(serData->timeSec), serData->timeNsec, serData->sampleRate);
       if (level>0) printf("     datastring=%s \n", serData->data);
       if (level>0 && serData->serial != NULL) FrVectDump(serData->serial, stdout, level);
       flag = 1;
      }
   }

 procData = FrProcDataFind(frame, varName);
 if (procData != NULL)
   {printf(" Processed Data: %s \n",procData->name);
    printf("    sampleRate=%5.1f\n", procData->sampleRate);
    if (procData->data != NULL)
       {printf(" Data: ");
        FrVectDump(procData->data, stdout, level);
       } 
    if (procData->aux != NULL)
       {printf(" Aux: ");
        FrVectDump(procData->aux, stdout, level);
       }
    flag = 1;
   }

 trigger = FrTrigDataFind(frame, varName);
 if (trigger != NULL)
   {printf(" TRIGGER: %s \n",trigger->name);
    printf("     comment: %s \n",trigger->comment);
    printf("     rvalue=%5.1f  proba=%5.1f   \n", trigger->rvalue, trigger->probability);
    printf("     statistics=%s \n", trigger->statistics);
    flag = 1;
   }

 simData = FrSimDataFind(frame, varName);
 if (simData != NULL) 
    {printf("Simulated Data: %s ", simData->name);
     if (simData->data != NULL) FrVectDump(simData->data, stdout, level); 
     flag = 1;
    }

 if (flag==0) printf("      %s not present in this frame \n",varName);
 return;
}



/*----------------------------------------------------- DumpAllAdc ----------*/

void DumpAllAdc(struct FrameH *frame, long level)
{struct FrAdcData *adcData;

 if (debug>10) printf("\n DumpAllAdc \n");

 if (!frame) return;

 if (frame->rawData)
   {
    adcData = frame->rawData->firstAdc;
    if (adcData == NULL) printf("       No adc data present in this frame \n");
    while (adcData != NULL)
      {printf(" ADC: %s \n",adcData->name);
       printf("     nBits=%i  sampleRate=%5.1f  crate=%i  channel=%i \n",
                    adcData->nBits, adcData->sampleRate, adcData->crate, adcData->channel);
       if (adcData->data != NULL) FrVectDump( adcData->data, stdout, level);
       adcData = adcData->next;
      }
   }
 return;
}




/*----------------------------------------------------- DumpAllSer ----------*/

void DumpAllSer(struct FrameH *frame, long level)
{struct FrSerData *serData;

 if (debug>10) printf("\n DumpAllSer \n");

 if (!frame) return;

 if (frame->rawData)
   {
    serData = frame->rawData->firstSer;
    if (serData == NULL) printf("       No sms data present in this frame \n");
    while(serData != NULL)
      {printf(" SMS: %s \n",serData->name);
       printf("     timeSec=%s  timeNsec=%i  sampleRate=%5.1f  \n",
                    FrStrGTime(serData->timeSec), serData->timeNsec, serData->sampleRate);
       if (level>0) printf("     datastring=%s \n", serData->data);
       if (level>0 && serData->serial != NULL) FrVectDump(serData->serial, stdout, level);
       serData = serData->next;
      }
   }
 return;
}




/*----------------------------------------------------- DumpAllProc ----------*/

void DumpAllProc(struct FrameH *frame, long level)
{struct FrProcData *procData;

 if (debug>10) printf("\n DumpAllProc \n");

 if (!frame) return;

 procData = frame->procData;
 if (frame->procData == NULL) printf("       No processed data present in this frame \n");
 while (procData != NULL)
   {printf(" Processed Data: %s \n",procData->name);
    printf("    sampleRate=%5.1f\n", procData->sampleRate);
    if (procData->data != NULL)
        {printf(" Data: ");
         FrVectDump(procData->data, stdout, level);
        } 
    if (procData->aux != NULL)
        {printf(" Aux: ");
         FrVectDump(procData->aux, stdout, level);
        }
     procData = procData->next;
   }
 return;
}



/*----------------------------------------------------- DumpAllTrig ----------*/

void DumpAllTrig(struct FrameH *frame, long level)
{struct FrTrigData *trigger;

 if (debug>10) printf("\n DumpAllTrig \n");

 if (!frame) return;

 trigger = frame->trigData;
 if (trigger == NULL) printf("       No trig data present in this frame \n");
 while (trigger != NULL)
   {printf(" TRIGGER: %s \n",trigger->name);
    printf("     comment: %s \n",trigger->comment);
    printf("     rvalue=%5.1f  proba=%5.1f   \n", trigger->rvalue, trigger->probability);
    printf("     statistics=%s \n", trigger->statistics);
    trigger = trigger->next;
   }
 return;
}




/*----------------------------------------------------- DumpAllStatic ----------*/

void DumpAllStatic(struct FrameH *frame, long level)
{
 if(frame->detectProc != NULL)
    {printf("Detector used for reconstruction:\n");
     FrDetectorDump(frame->detectProc, stdout, level);
    }
 if(frame->detectSim != NULL)
    {printf("Detector used for simulation:\n");
     FrDetectorDump(frame->detectSim, stdout, level);
    }
 return;
}




/*----------------------------------------------------- DumpAllSim ----------*/

void DumpAllSim(struct FrameH *frame, long level)
{struct FrSimData *simData;

 simData = frame->simData;
 while(simData != NULL) 
    {printf("Simulated Data: %s ", simData->name);
     if (simData->data != NULL) FrVectDump(simData->data, stdout, level); 
     simData = simData->next;
    }
 return;
}




/*---------------------------------------------------- my_strcpy ---------*/
char *my_strcpy(char *dummy, char *str)
{
 if (debug>200) printf("\nmy_strcpy \n");

 if (!str) return(NULL);

 if (dummy) free(dummy);
 dummy = (char *) calloc(strlen(str)+1,sizeof(char));
 strcpy(dummy, str);

 return(dummy);
}




/*----------------------------------------------------- is_a_dir ----------*/

long is_a_dir(const char *str)
{DIR *fd;

 fd = opendir(str);
 if (fd)
   {closedir(fd);
    return 1;
   }
 return 0;
}
